#!/usr/bin/env python
# -*- coding: utf-8 -*-
#-----------------------------------------------------------------------------
# Copyright (c) 2015, IBM Corp.
# All rights reserved.
#
# Distributed under the terms of the BSD Simplified License.
#
# The full license is in the LICENSE file, distributed with this software.
#-----------------------------------------------------------------------------

"""
Test module for IdaDataFrameObjects
"""
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
standard_library.install_aliases()


class Test_aggregation(object):

    # Don't forget to test the recursive way too

    def test_idadf_aggregation_1_column_add_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mul_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_div_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_sub_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_floordiv_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mod_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_neg_int(idadf, df):
        pass

    def test_idadf_aggregation_1_column_pow_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_add_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mul_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_div_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_sub_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_floordiv_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mod_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_neg_int(idadf, df):
        pass

    def test_idadf_aggregation_X_column_pow_int(idadf, df):
        pass


    def test_idadf_aggregation_1_column_add_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mul_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_div_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_sub_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_floordiv_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_mod_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_neg_ida(idadf, df):
        pass

    def test_idadf_aggregation_1_column_pow_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_add_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mul_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_div_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_sub_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_floordiv_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_mod_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_neg_ida(idadf, df):
        pass

    def test_idadf_aggregation_X_column_pow_ida(idadf, df):
        pass

    def test_idadf_aggregation_error(idadf, df):
        pass


